<?php

namespace Drupal\entity_theme_engine;


use Drupal\Core\Entity\EntityInterface;
use Drupal\entity_theme_engine\Entity\EntityWidget;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\field\Entity\FieldConfig;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * Widget Helper.
 */
class EntityWidgetService {

  /**
   * 
   * @var \Symfony\Component\Serializer\SerializerInterface
   */
  public $serializer;
  
  /**
   * Constructs.
   * 
   * @param SerializerInterface $serializer
   */
  public function __construct(SerializerInterface $serializer) {
    $this->serializer = $serializer;
  }

  /**
   * Entity view alter.
   * @param array $build
   * @param EntityInterface $entity
   * @param string $display
   */
  public function entityViewAlter(array &$build, EntityInterface $entity, string $display = 'default') {
    if($display == 'full'){
      $display = 'default';
    }
    if($widget = $this->getWidget($entity, $display)) {
      $build = $this->renderEntity($build, $widget, $entity);
    }
  }
  /**
   * Render widget entity.
   * 
   * @param array $build
   * @param EntityWidget $widget
   * @param EntityInterface $entity
   * @param string $display
   * @return array
   */
  public function renderEntity(array $build, EntityWidget $widget, EntityInterface $entity) {
//     $variables = $this->processManager->process($build, $widget, $entity);

    $variables = $this->getRenderVariables($build, $widget, $entity);
    
    $render = [];
    $render['content'] = [
      '#type' => 'inline_template',
      '#template' => $widget->getTemplate(),
      '#context' => $variables,
    ];
    if($library = $widget->getLibrary()) {
      $render['#attached']['library'] = [$library];
    }
    $this->addSpacer($render['content']);
    return $render;
  }

  /**
   * Get widget.
   * @param EntityInterface $entity
   * @param string $display
   * @return EntityWidget|NULL
   */
  public function getRenderVariables(array $build, EntityWidget $widget, EntityInterface $entity) {
    $variables = [
      'id' => $entity->id(),
      'uuid' => $entity->uuid(),
      'entity_label' => $entity->label(),
      'entity_url' => $entity->toUrl()->toString(),
      'entity_type' => $entity->getEntityTypeId(),
    ];
    if($entity instanceof FieldableEntityInterface) {
      $fields = $entity->getFieldDefinitions();
      foreach ($fields as $field_name => $field_def) {
        if($field_def instanceof FieldConfig) {
          $field_type = $field_def->getType();
          $context = [
            'field_name' => $field_name,
            'field_type' => $field_type,
            'field_definition' => $field_def,
            'entity_widget' => $widget,
            'entity' => $entity,
          ];
          if($field_def->getFieldStorageDefinition()->getCardinality() == 1) {
            if($entity->get($field_name)->isEmpty()){
              $variables[$field_name] = NULL;
            }else {
              $variables[$field_name] = $this->serializer->serialize($entity->get($field_name)->get(0), 'twig_variable', $context);
            }
          }else {
            foreach ($entity->get($field_name) as $item) {
              $variables[$field_name]['items'][] = $this->serializer->serialize($item, 'twig_variable', $context);
            }
          }
        }
      }
    }
    foreach ($variables as $key => $value) {
      if(isset($build[$key])) {
        $variables[$key]['#original'] = $build[$key];
      }
    }
    \Drupal::moduleHandler()->alter('entity_widget_variables', $variables, $widget, $entity);
    return $variables;
  }
  /**
   * Get widget.
   * @param EntityInterface $entity
   * @param string $display
   * @return EntityWidget|NULL
   */
  public function getWidget(EntityInterface $entity, string $display) {
    $widgets = $this->getAllWidgets();
    $active_theme = \Drupal::theme()->getActiveTheme()->getName();
    $keys = [
      "{$active_theme}:{$entity->getEntityTypeId()}:{$entity->bundle()}:{$display}",
      "{$entity->getEntityTypeId()}:{$entity->bundle()}:{$display}",
    ];
    foreach ($keys as $key) {
      if(isset($widgets[$key])) {
        return $widgets[$key];
      }
    }
    return NULL;
  }
  /**
   * Get all widgets.
   * @return array|mixed|mixed[]|\Drupal\Core\Entity\EntityInterface
   */
  public function getAllWidgets() {
    $cache = &drupal_static(__METHOD__, []);
    if(empty($cache)) {
      $widgets = \Drupal::entityTypeManager()->getStorage('entity_widget')->loadMultiple();
      foreach ($widgets as $widget) {
        if(!empty($widget->getTheme())) {
          $cache["{$widget->getTheme()}:{$widget->getWidgetEntityType()}:{$widget->getBundle()}:{$widget->getDisplay()}"] = $widget;
        }else {
          $cache["{$widget->getWidgetEntityType()}:{$widget->getBundle()}:{$widget->getDisplay()}"] = $widget;
        }
      }
    }
    return $cache;
  }
  /**
   * Add spacer.
   * @param array $build
   */
  protected function addSpacer(array &$build)
  {
    $name = false;
    $variables = $build['#context'];
    if(!empty($variables['spacer']['value'])) {
      $name = $variables['spacer']['value'];
    }
    if($name) {
      switch ($name) {
        case 'large_bottom':
          $build['#suffix'] = '<div class="spacer-lg"></div>';
          break;
        case 'large_top':
          $build['#prefix'] = '<div class="spacer-lg"></div>';
          break;
        case 'large_both':
          $build['#prefix'] = '<div class="spacer-lg"></div>';
          $build['#suffix'] = '<div class="spacer-lg"></div>';
          break;
        case 'medium_bottom':
          $build['#suffix'] = '<div class="spacer-md"></div>';
          break;
        case 'medium_top':
          $build['#prefix'] = '<div class="spacer-md"></div>';
          break;
        case 'medium_both':
          $build['#prefix'] = '<div class="spacer-md"></div>';
          $build['#suffix'] = '<div class="spacer-md"></div>';
          break;
        case 'small_bottom':
          $build['#suffix'] = '<div class="spacer-sm"></div>';
          break;
        case 'small_top':
          $build['#prefix'] = '<div class="spacer-sm"></div>';
          break;
        case 'small_both':
          $build['#prefix'] = '<div class="spacer-sm"></div>';
          $build['#suffix'] = '<div class="spacer-sm"></div>';
          break;
        default :
          break;
      }
    }
  }
}
